<?php

namespace Drupal\entity_information\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Link;
use Drupal\entity_information\EntityInformationManager;
use Drupal\entity_information\Form\SettingsForm;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides route responses for entity information.
 */
class EntityInformationController extends ControllerBase {

  /**
   * The entity information manager service.
   *
   * @var \Drupal\entity_information\EntityInformationManager|null
   */
  protected $entityInformationManager;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    /** @var static $controller */
    $controller = parent::create($container);
    $controller->setEntityInformationManager($container->get('plugin.manager.entity_information'));
    return $controller;
  }

  /**
   * Retrieves the entity information manager service.
   *
   * @return \Drupal\entity_information\EntityInformationManager
   *   The entity information manager service.
   */
  public function getEntityInformationManager(): EntityInformationManager {
    return $this->entityInformationManager ?: \Drupal::service('plugin.manager.entity_information');;
  }

  /**
   * Sets the entity information manager service.
   *
   * @param \Drupal\entity_information\EntityInformationManager $entityInformationManager
   *   The entity information manager service.
   *
   * @return $this
   */
  public function setEntityInformationManager(EntityInformationManager$entityInformationManager): self {
    $this->entityInformationManager = $entityInformationManager;
    return $this;
  }

  /**
   * Renders the entity information plugins.
   *
   * @return array
   *   The render array.
   */
  public function render(): array {
    // Get the entity we are building the information plugins for.
    $entity = $this->getEntityInformationManager()->pluggedEntity();

    // Get plugged entity types.
    $plugged_entity_types = $this->getEntityInformationManager()->getPluggedEntityTypes();

    // Get plugin configuration te determine which plugins are enabled.
    $enabled_plugins = $this->config(SettingsForm::SETTINGS)->get('enabled_plugins');

    // Get all available entity information plugins for the current entity.
    $plugins = [];
    if (isset($plugged_entity_types[$entity->getEntityTypeId()][$entity->bundle()])) {
      foreach ($plugged_entity_types[$entity->getEntityTypeId()][$entity->bundle()]['plugins'] as $plugin_name =>$plugin) {
        if (isset($enabled_plugins[$plugin_name]) && $enabled_plugins[$plugin_name] !== 0) {
          $plugins[$plugin_name] = $plugin;
        }
      }
    }

    // Return message if no plugins were found.
    if (count($plugins) === 0) {
      return [
        '#type' => 'markup',
        '#title' => $this->t('Entity information'),
        '#markup' => $this->t('No entity information available. With the right permissions you can enable entity information containers on the @page.', [
          '@page' => Link::createFromRoute('configuration page', 'entity_information.settings')->toString(),
        ]),
      ];
    }

    // Sort plugins by weight.
    uasort($plugins, function($a, $b) {
      return $a['weight'] <=> $b['weight'];
    });

    // Initiate build array.
    $build = [];

    foreach ($plugins as $plugin_id => $plugin) {
      if ($plugin = $this->getEntityInformationManager()->createInstance($plugin_id, [])) {
        $definitions = $plugin->getPluginDefinition();

        $build[$plugin_id]['block'] = [
          '#type' => 'details',
          '#title' => $definitions['label'] ?? $this->t('Information'),
          '#open' => $definitions['open'] ?? TRUE,
        ];

        $build[$plugin_id]['block']['body'] = $plugin->view($entity);
      }
    }

    return $build;
  }

}
