<?php

namespace Drupal\entity_information\Plugin\EntityInformation;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\entity_information\Plugin\EntityInformationBase;
use Drupal\entity_information\Plugin\EntityInformationInterface;

/**
 * Builds path alias information.
 *
 * @EntityInformation(
 *   id = "entity_information_path_alias",
 *   label = @Translation("Path Alias Information"),
 *   bundles = {
 *     "node.*",
 *   },
 *   weight = 1,
 *   open = TRUE
 * )
 */
class EntityInformationPathAlias extends EntityInformationBase implements EntityInformationInterface {

  /**
   * {@inheritdoc}
   */
  public function view(ContentEntityInterface $entity): array {
    // We can only show the aliases when the path module is enabled.
    if (!$this->moduleHandler()->moduleExists('path')) {
      $this->setNoResultsMessage($this->t('The Path module needs to be enabled to show the path alias information.'));
      return $this->noResultsResponse();
    }

    // Get all path aliases for this entity.
    $path_aliases = $this->entityTypeManager()->getStorage('path_alias')->loadByProperties([
      'path' => '/' . $entity->toUrl()->getInternalPath(),
    ]);

    // Show a message when no alias is found.
    /** @var \Drupal\node\Entity\NodeType $bundle */
    $bundle = $entity->get('type')->entity;
    if (empty($path_aliases)) {
      $this->setNoResultsMessage($this->t('No path alias found for this :bundle.',  [':bundle' => $bundle->label()]));
      return $this->noResultsResponse();
    }

    // Build the table header.
    $header = [];
    $header[] = [
      'data' => $this->t('Path Alias'),
      'field' => 'alias',
    ];
    $header[] = [
      'data' => $this->t('Language'),
      'field' => 'langcode',
    ];
    $header[] = [
      'data' => $this->t('Operations'),
    ];

    // Build the table rows.
    $destination = $this->getDestinationArray();
    $rows = [];
    /** @var \Drupal\path_alias\Entity\PathAlias[] $path_aliases */
    foreach ($path_aliases as $path_alias) {
      $row = [];
      $url = $entity->toUrl('canonical', ['language' => $entity->language()]);
      $row['data']['alias'] = Link::fromTextAndUrl($url->toString(), $url);
      $row['data']['language_name'] = $path_alias->language() ? $path_alias->language()->getName() : 'Und';
      $operations = [];
      $operations['edit'] = [
        'title' => $this->t('Edit'),
        'url' => Url::fromRoute('entity.path_alias.edit_form', ['path_alias' => $path_alias->id()], ['query' => $destination]),
      ];
      $operations['delete'] = [
        'title' => $this->t('Delete'),
        'url' => Url::fromRoute('entity.path_alias.delete_form', ['path_alias' => $path_alias->id()], ['query' => $destination]),
      ];
      $row['data']['operations'] = [
        'data' => [
          '#type' => 'operations',
          '#links' => $operations,
        ],
      ];
      $rows[] = $row;
    }

    // Setup and return the build array for this entity information container.
    return [
      'path_aliases' => [
        'label' => [
          '#type' => 'item',
          '#title' => $this->t('The following path aliases are available for this :bundle:', [':bundle' => $bundle->label()]),
        ],
        'overview' => [
          '#type' => 'table',
          '#header' => $header,
          '#rows' => $rows,
        ],
      ]
    ];
  }

}
